"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const turbo = require("turbo-http-clusterable");
const FMW = require("find-my-way");
const qs = require("querystring");
const url = require("url");
var HttpMethod;
(function (HttpMethod) {
    HttpMethod["GET"] = "GET";
    HttpMethod["POST"] = "POST";
    HttpMethod["PUT"] = "PUT";
    HttpMethod["OPTIONS"] = "OPTIONS";
    HttpMethod["DELETE"] = "DELETE";
    HttpMethod["PATCH"] = "PATCH";
})(HttpMethod = exports.HttpMethod || (exports.HttpMethod = {}));
class SlimJim {
    constructor() {
        this.router = FMW();
        this.server = turbo.createServer((req, res) => {
            req.headers = indexHeaders(req._options.headers);
            this.router.lookup(req, res);
        });
    }
    start(port, address, callback) {
        this.server.listen(port, address, callback);
    }
    get(path, handler) {
        this.endpoint(HttpMethod.GET, path, handler);
    }
    post(path, handler) {
        this.endpoint(HttpMethod.POST, path, handler);
    }
    put(path, handler) {
        this.endpoint(HttpMethod.PUT, path, handler);
    }
    patch(path, handler) {
        this.endpoint(HttpMethod.PATCH, path, handler);
    }
    options(path, handler) {
        this.endpoint(HttpMethod.OPTIONS, path, handler);
    }
    delete(path, handler) {
        this.endpoint(HttpMethod.DELETE, path, handler);
    }
    endpoint(method, path, handler) {
        this.router.on(method, path, (req, res, params) => {
            let rawBody = null;
            const headers = req.headers;
            req.ondata = (buffer, start, length) => {
                rawBody = buffer.toString("utf8", start, start + length);
            };
            req.onend = () => {
                const parsedUrl = url.parse(req.url, true);
                const handlerReq = {
                    body: attemptParse(headers["content-type"], rawBody),
                    method,
                    params,
                    url: parsedUrl.path,
                    rawBody,
                    headers,
                    query: parsedUrl.query,
                };
                handler(handlerReq, (error, handlerRes) => {
                    if (error != null) {
                        res.statusCode = 500;
                        res.end(null);
                    }
                    res.statusCode = handlerRes.statusCode;
                    const headerKeys = Object.keys(handlerRes.headers);
                    for (let i = 0; i < headerKeys.length; i++) {
                        const key = headerKeys[i];
                        res.setHeader(key, handlerRes.headers[key]);
                    }
                    const resContentType = handlerRes.headers["Content-Type"] || "text/plain";
                    const content = serialize(resContentType, handlerRes.body);
                    res.end(content);
                });
            };
        });
    }
    stop(callback) {
        this.server.close(callback);
    }
}
exports.SlimJim = SlimJim;
function serialize(contentType, data) {
    if (data == null) {
        return "";
    }
    else if (typeof data !== "string") {
        return JSON.stringify(data);
    }
    else {
        return data;
    }
}
function indexHeaders(headerArr) {
    const retObj = {};
    const len = headerArr.length;
    for (let i = 0; i < len; i += 2) {
        retObj[headerArr[i]] = headerArr[i + 1];
    }
    return retObj;
}
function attemptParse(contentType, obj) {
    if (contentType == null) {
        return obj;
    }
    try {
        switch (contentType) {
            case "application/json":
                return JSON.parse(obj);
            case "application/x-www-form-urlencoded":
                return qs.parse(obj);
            default:
                return obj;
        }
    }
    catch (err) {
        return obj;
    }
}
